@@ -5,7 +5,7 @@
 # Author    : Chris Jones
 # Email     : crj341@student.bham.ac.uk
 # Date      : 26/06/2020
-# Version   : 0.1.3
+# Version   : 0.1.14
 
 import  numpy                   as      np
 import  plotly.graph_objects    as      go
@@ -142,6 +142,10 @@ class AFM:
         Funciton to produce a box plot comparing adhesion values for multiple experiments.
         A separate instance of the AFM class containing the data for each should be
         given as the inputs
+    overlay_adhesion_violin:
+        Funciton to produce a violin plot comparing adhesion values for multiple experiments.
+        A separate instance of the AFM class containing the data for each should be
+        given as the inputs
     overlay_adhesion_bar:
         Funciton to produce a bar chart comparing adhesion values for multiple experiments.
         A separate instance of the AFM class containing the data for each should be
@@ -158,6 +162,10 @@ class AFM:
         Funciton to produce a box plot comparing modulus values for multiple experiments.
         A separate instance of the AFM class containing the data for each should be
         given as the inputs
+    overlay_modulus_violin:
+        Funciton to produce a violin plot comparing modulus values for multiple experiments.
+        A separate instance of the AFM class containing the data for each should be
+        given as the inputs
     overlay_modulus_bar:
         Funciton to produce a bar chart comparing modulus values for multiple experiments.
         A separate instance of the AFM class containing the data for each should be
@@ -260,10 +268,11 @@ class AFM:
 
         file_no_ext = file[:-3]
 
-        ext = '000'
-        i = 0
-        j = 0
-        k = 0
+        i = int(file[-3])
+        j = int(file[-2])
+        k = int(file[-1])
+
+        ext = str(i) + str(j) + str(k)
 
         _filelist = list('')
 
@@ -284,8 +293,6 @@ class AFM:
 
         self.filelist = _filelist
 
-        
-
     def nanoscope_params(
         self,
         nanoscope_version = 6
@@ -302,7 +309,7 @@ class AFM:
             exit()
 
         filename = self.filelist[0]
-        data = open(filename,"r")
+        data = open(filename, "r", errors='ignore')
 
         nanoscope_chk = data.readline().strip()
         if not nanoscope_chk.startswith('\*Force file list'):
@@ -399,7 +406,10 @@ class AFM:
         self.def_sens = float(_def_sens)
         self.spr_const = float(_spr_const)
 
-    def nanoscope_read(self):
+    def nanoscope_read(
+        self,
+        adjusted_tip_position = False
+        ):
         '''
         Function to import raw force curve data
         '''
@@ -419,7 +429,17 @@ class AFM:
             _retract_curve = np.fromfile(file = data, dtype = 'int16', count = self.samps_line) * self.zscale * self.spr_const * self.def_sens
 
             hscale = (self.zsens * self.ramp_size * 2 ** 16 / self.samps_line ) / 2
-            _z_position = hscale * np.arange(1, self.samps_line + 1)
+            tip_position = hscale * np.arange(1, self.samps_line + 1)
+
+            if adjusted_tip_position is True:
+
+                _z_approach = tip_position + self.spr_const * _approach_curve
+                _z_retract = tip_position + self.spr_const * _retract_curve
+
+                _z_position = np.concatenate((_z_approach, _z_retract), axis=0)
+
+            else:
+                _z_position = np.concatenate((tip_position, tip_position), axis=0)
 
             self.approach_raw[:,i] = np.array(_approach_curve)
             self.retract_raw[:,i] = np.array(_retract_curve)
@@ -436,8 +456,8 @@ class AFM:
         plot must be given as an input <.plot_raw(n)>
         '''
         fig = go.Figure()
-        fig.add_trace(go.Scatter(y = self.approach_raw[:,curve_number], x = self.z_position_raw[:,curve_number], mode='lines', name = 'Approach curve (raw)'))
-        fig.add_trace(go.Scatter(y = self.retract_raw[:,curve_number], x = self.z_position_raw[:,curve_number], mode='lines', name = 'Retract curve (raw)'))
+        fig.add_trace(go.Scatter(y = self.approach_raw[:,curve_number], x = self.z_position_raw[:,curve_number][0:int(self.samps_line/2)], mode='lines', name = 'Approach curve (raw)'))
+        fig.add_trace(go.Scatter(y = self.retract_raw[:,curve_number], x = self.z_position_raw[:,curve_number][int(self.samps_line/2):self.samps_line], mode='lines', name = 'Retract curve (raw)'))
         fig.update_layout(
             yaxis_title_text='Force (nN)',
             xaxis_title_text='Tip separation (nm)',
@@ -476,10 +496,11 @@ class AFM:
 
             baseline_start = int(len(_z) * start_pos)
             baseline_end = int(len(_z) * end_pos)
-            baseline_region = np.arange(baseline_start,baseline_end)
+            approach_baseline_region = np.arange(baseline_start,baseline_end)
+            retract_baseline_region = np.arange(baseline_start + self.samps_line, baseline_end + self.samps_line)
 
-            _approach_region = _approach[baseline_region]
-            _retract_region = _retract[baseline_region]
+            _approach_region = _approach[approach_baseline_region]
+            _retract_region = _retract[retract_baseline_region]
             _z_region = _z[baseline_region]
 
             _approach_mean = np.mean(_approach_region)
@@ -540,8 +561,8 @@ class AFM:
         plot must be given as an input <.plot_adjusted(n)>
         '''
         fig = go.Figure()
-        fig.add_trace(go.Scatter(y = self.approach[:,curve_number], x = self.z_position[:,curve_number], mode='lines', name = 'Approach curve'))
-        fig.add_trace(go.Scatter(y = self.retract[:,curve_number], x = self.z_position[:,curve_number], mode='lines', name = 'Retract curve'))
+        fig.add_trace(go.Scatter(y = self.approach[:,curve_number], x = self.z_position[:,curve_number][0:int(self.samps_line/2)], mode='lines', name = 'Approach curve'))
+        fig.add_trace(go.Scatter(y = self.retract[:,curve_number], x = self.z_position[:,curve_number][int(self.samps_line/2):self.samps_line], mode='lines', name = 'Retract curve'))
         fig.update_layout(
             yaxis_title_text='Force (nN)',
             xaxis_title_text='Tip separation (nm)',
@@ -562,14 +583,14 @@ class AFM:
                         visible=False,
                         name= 'Approach ' + str(step),
                         y = self.approach[:, step],
-                        x = self.z_position[:, step]
+                        x = self.z_position[:, step][0:int(self.samps_line/2)]
                     ))
 
             trace_2.append(go.Scatter(
                         visible=False,
                         name= 'Retract ' + str(step),
                         y = self.retract[:, step],
-                        x = self.z_position[:, step]
+                        x = self.z_position[:, step][int(self.samps_line/2):self.samps_line]
                     ))
 
         fig = go.Figure(data = trace_1 + trace_2)
@@ -619,17 +640,19 @@ class AFM:
         i = 0
         for curve_number in curves:
 
-            self.approach = np.delete(self.approach, curve_number, axis = 1)
-            self.retract = np.delete(self.retract, curve_number, axis = 1)
-            self.z_position = np.delete(self.z_position, curve_number, axis = 1)
+            curve_int = int(curve_number)
+
+            self.approach = np.delete(self.approach, curve_int, axis = 1)
+            self.retract = np.delete(self.retract, curve_int, axis = 1)
+            self.z_position = np.delete(self.z_position, curve_int, axis = 1)
 
             try:
-                self.adhesion = np.delete(self.adhesion, curve_number, axis = 1)
+                self.adhesion = np.delete(self.adhesion, curve_int, axis = 1)
             except:
                 pass
 
             try:
-                self.modulus = np.delete(self.modulus, curve_number, axis = 1)
+                self.modulus = np.delete(self.modulus, curve_int, axis = 1)
             except:
                 pass
 
@@ -781,6 +804,28 @@ class AFM:
             
         fig.show()
 
+    def overlay_adhesion_violin(
+        self,
+        *all_adhesion
+    ):
+        '''
+        Funciton to produce a violin plot comparing adhesion values for multiple experiments.
+        A separate instance of the AFM class containing the data for each should be
+        given as the inputs
+        '''
+        fig = go.Figure()
+
+        for single in all_adhesion:
+            fig.add_trace(go.Violin(x = single.adhesion,
+            name = single.run_name
+        ))
+
+        fig.update_layout(
+                xaxis_title_text='Adhesion (nN)'
+            )
+            
+        fig.show()
+
     def overlay_adhesion_bar(
         self,
         *all_adhesion
@@ -816,8 +861,11 @@ class AFM:
     def calc_modulus(
         self,
         poisson_ratio = 0.5,
-        indenter_radius = 10 * 10e-9,
-        plot_hist = False
+        indenter_radius = 10 * 1e-9,
+        plot_hist = False,
+        soft_indenter = False,
+        indenter_poisson_ratio = 0.5,
+        indenter_modulus = 1000
     ):
         '''
         Funciton to calculate the modulus value for each curve using the Hertz model.
@@ -826,12 +874,17 @@ class AFM:
         '''
         self.modulus = np.zeros(self.z_position.shape[1])
 
+        if soft_indenter is True:
+            indenter_term = (1 - indenter_poisson_ratio ** 2) / (indenter_modulus * 1e6)
+        else:
+            indenter_term = 0
+
         for curve_number in range(0, self.z_position.shape[1]):
             _approach = self.approach[:,curve_number]
             _z = self.z_position[:,curve_number]
 
-            force = _approach[0] * 10e-9
-            depth = _z[0] * 10e-9
+            force = _approach[0] * 1e-9
+            depth = _z[0] * 1e-9
             ratio = depth / indenter_radius
 
             term_1 = (4/3)
@@ -839,11 +892,10 @@ class AFM:
             term_3 = 1 / force
             term_4 = np.sqrt(indenter_radius)
             term_5 = np.absolute(depth) ** (3/2)
-            # term_6 = (1 - (1/10) * ratio - (1/840) * ratio ** 2 + (11/15120) * ratio ** 3 + (1357/6652800) * ratio ** 4)
 
-            terms = term_1 * term_2 * term_3 * term_4 * term_5 #* term_6
+            terms = (term_1* term_3 * term_4 * term_5 - indenter_term) * term_2 
 
-            self.modulus[curve_number] = (1 / terms) * 10e-6
+            self.modulus[curve_number] = (1 / terms) * 1e-6
         
         if plot_hist is True:
             fig = go.Figure(data=[go.Histogram(x = self.modulus)])
@@ -908,6 +960,28 @@ class AFM:
             
         fig.show()
 
+    def overlay_modulus_violin(
+        self,
+        *all_modulus
+    ):
+        '''
+        Funciton to produce a violin plot comparing modulus values for multiple experiments.
+        A separate instance of the AFM class containing the data for each should be
+        given as the inputs
+        '''
+        fig = go.Figure()
+
+        for single in all_modulus:
+            fig.add_trace(go.Violin(x = single.modulus,
+            name = single.run_name,
+        ))
+
+        fig.update_layout(
+                xaxis_title_text='Modulus (MPa)'
+            )
+            
+        fig.show()
+
     def overlay_modulus_bar(
         self,
         *all_modulus
